@@ -27,6 +27,7 @@ def call_if_callable(key, widget, data):
         return attr(widget, data)
     return attr
 
+@managedprops('required', 'time', 'locale', 'tzinfo')
 def datetime_extractor(widget, data):
     time = None
     if call_if_callable('time', widget, data):
@@ -67,15 +68,19 @@ def render_datetime_input(widget, data, date, time):
             time = ''
         if not time and data.request:
             time = data.request.get('%s.time' % widget.dottedpath)
-        timeinput = tag('input', **{
+        attrs = {
             'type': 'text',
             'value': time,
             'name_': '%s.time' % widget.dottedpath,
             'id': cssid(widget, 'input', 'time'),
             'size': 5,
-        })
+            'disabled': widget.attrs['disabled'] and 'disabled' or None,
+        }
+        if widget.attrs['timepicker'] and not widget.attrs['disabled']:
+            attrs['class_'] = widget.attrs['timepicker_class']
+        timeinput = tag('input', **attrs)
     additional_classes = []
-    if widget.attrs['datepicker']:
+    if widget.attrs['datepicker'] and not widget.attrs['disabled']:
         additional_classes.append(widget.attrs['datepicker_class'])
     attrs = {
         'type': 'text',
@@ -84,11 +89,13 @@ def render_datetime_input(widget, data, date, time):
         'id': cssid(widget, 'input'),
         'class_': cssclasses(widget, data, additional=additional_classes),
         'size': 10,
+        'disabled': widget.attrs['disabled'] and 'disabled' or None,
     }
     return tag('input', **attrs) + timeinput
 
 
-@managedprops('locale', *css_managed_props)
+@managedprops('locale', 'delimiter', 'time', 'disabled', 'timepicker',
+              'timepicker_class', 'datepicker', 'datepicker_class', *css_managed_props)
 def datetime_edit_renderer(widget, data):
     locale = call_if_callable('locale', widget, data)
     delim = call_if_callable('delimiter', widget, data)
@@ -106,15 +113,21 @@ def datetime_edit_renderer(widget, data):
         date = fetch_value(widget, data)
     return render_datetime_input(widget, data, date, time)
 
-
-def datetime_display_renderer(widget, data):
-    if not data.value:
+@managedprops('format', 'class')
+def datetime_display_renderer(widget, data, value=None):
+    """Note: This renderer function optionally accepts value as parameter,
+    which is used in favor of data.value if defined. Thus it can be used as
+    utility function inside custom blueprints with the need of datetime
+    display rendering.
+    """
+    value = value and value or data.value
+    if not value:
         return u''
     format = widget.attrs['format']
     if callable(format):
         value = format(widget, data)
     else:
-        value = data.value.strftime(format)
+        value = value.strftime(format)
     attrs = {
         'id': cssid(widget, 'display'),
         'class_': 'display-%s' % widget.attrs['class']
@@ -141,10 +154,20 @@ factory.defaults['datetime.class'] = 'datetime'
 factory.defaults['datetime.required_class'] = 'required'
 
 factory.defaults['datetime.datepicker_class'] = 'datepicker'
+factory.doc['props']['datetime.time'] = \
+"""jquery.ui datepicker binds to this class.
+"""
+
+factory.defaults['datetime.timepicker_class'] = 'timepicker'
+factory.doc['props']['datetime.timepicker_class'] = \
+"""jquery.ui timepicker binds to this class.
+"""
+
+factory.defaults['datetime.disabled'] = False
 
 factory.defaults['datetime.datepicker'] = False
-factory.doc['props']['datetime.datepicker'] = \
-"""Flag whether datepicker should be rendered.
+factory.doc['props']['datetime.datepicker_class'] = \
+"""Flag whether date picker is enabled.
 """
 
 factory.defaults['datetime.time'] = False
@@ -155,6 +178,16 @@ factory.doc['props']['datetime.time'] = \
 a boolean.
 """
 
+factory.defaults['datetime.timepicker'] = False
+factory.doc['props']['datetime.timepicker'] = \
+"""Flag whether time picker is enabled.
+"""
+
+factory.defaults['datetime.datepicker'] = False
+factory.doc['props']['datetime.datepicker'] = \
+"""Flag whether date picker is enabled.
+"""
+
 factory.defaults['datetime.tzinfo'] = None
 factory.doc['props']['datetime.tzinfo'] = \
 """Python datetime tzinfo object.
